@@ -1,14 +1,17 @@
 """Library to handle connection with Tibber API."""
 import asyncio
+import datetime as dt
 import logging
 
 import aiohttp
 import async_timeout
-from gql import gql
-from graphql.language.printer import print_ast
+import pytz
+from dateutil.parser import parse
 from graphql_subscription_manager import SubscriptionManager
 
-DEFAULT_TIMEOUT = 10
+from .const import RESOLUTION_HOURLY
+
+DEFAULT_TIMEOUT = 20
 DEMO_TOKEN = "d1007ead2dc84a2b82f0de19451c5fb22112f7ae11d19bf2bedb224a003ff74a"
 API_ENDPOINT = "https://api.tibber.com/v1-beta/gql"
 SUB_ENDPOINT = "wss://api.tibber.com/v1-beta/gql/subscriptions"
@@ -22,7 +25,11 @@ class Tibber:
     # pylint: disable=too-many-instance-attributes
 
     def __init__(
-        self, access_token=DEMO_TOKEN, timeout=DEFAULT_TIMEOUT, websession=None
+        self,
+        access_token=DEMO_TOKEN,
+        timeout=DEFAULT_TIMEOUT,
+        websession=None,
+        time_zone=None,
     ):
         """Initialize the Tibber connection."""
         if websession is None:
@@ -36,6 +43,7 @@ class Tibber:
             self.websession = websession
         self._timeout = timeout
         self._access_token = access_token
+        self.time_zone = time_zone or pytz.utc
         self._name = None
         self._home_ids = []
         self._all_home_ids = []
@@ -76,8 +84,7 @@ class Tibber:
 
     async def _execute(self, document, variable_values=None, retry=2):
         """Execute gql."""
-        query_str = print_ast(document)
-        payload = {"query": query_str, "variables": variable_values or {}}
+        payload = {"query": document, "variables": variable_values or {}}
 
         post_args = {
             "headers": {"Authorization": "Bearer " + self._access_token},
@@ -97,9 +104,7 @@ class Tibber:
                 return await self._execute(document, variable_values, retry - 1)
             raise
         except asyncio.TimeoutError as err:
-            _LOGGER.error(
-                "Timed out when connecting to Tibber: %s ", err, exc_info=True
-            )
+            _LOGGER.error("Timed out when connecting to Tibber: %s ", err)
             if retry > 0:
                 return await self._execute(document, variable_values, retry - 1)
             raise
@@ -116,8 +121,7 @@ class Tibber:
 
     async def update_info(self, *_):
         """Update home info async."""
-        query = gql(
-            """
+        query = """
         {
           viewer {
             name
@@ -130,7 +134,6 @@ class Tibber:
           }
         }
         """
-        )
 
         res = await self._execute(query)
         if res is None:
@@ -191,8 +194,7 @@ class Tibber:
 
     async def send_notification(self, title, message):
         """Send notification."""
-        query = gql(
-            """
+        query = """
         mutation{
           sendPushNotification(input: {
             title: "%s",
@@ -202,8 +204,9 @@ class Tibber:
             pushedToNumberOfDevices
           }
         }
-        """
-            % (title, message)
+        """ % (
+            title,
+            message,
         )
 
         res = await self.execute(query)
@@ -237,6 +240,7 @@ class TibberHome:
         self.info = {}
         self._subscription_id = None
         self._data = None
+        self.last_data_timestamp = None
 
     def sync_update_info(self):
         """Update current price info."""
@@ -246,7 +250,7 @@ class TibberHome:
 
     async def update_info(self):
         """Update current price info async."""
-        query = gql(
+        query = (
             """
         {
           viewer {
@@ -306,6 +310,7 @@ class TibberHome:
         """
             % self._home_id
         )
+
         self.info = await self._tibber_control.execute(query)
 
     def sync_update_current_price_info(self):
@@ -316,7 +321,7 @@ class TibberHome:
 
     async def update_current_price_info(self):
         """Update current price info async."""
-        query = gql(
+        query = (
             """
         {
           viewer {
@@ -359,7 +364,7 @@ class TibberHome:
 
     async def update_price_info(self):
         """Update price info async."""
-        query = gql(
+        query = (
             """
         {
           viewer {
@@ -446,7 +451,7 @@ class TibberHome:
             sub = self.info["viewer"]["home"]["currentSubscription"]["status"]
         except (KeyError, TypeError):
             return False
-        return sub == "running"
+        return sub in ["running", "awaiting market"]
 
     @property
     def has_real_time_consumption(self):
@@ -505,7 +510,7 @@ class TibberHome:
             _LOGGER.error("Already subscribed.")
             return
         await self._tibber_control.rt_connect(loop)
-        document = gql(
+        document = (
             """
             subscription{
               liveMeasurement(homeId:"%s"){
@@ -522,20 +527,20 @@ class TibberHome:
                 voltagePhase1
                 voltagePhase2
                 voltagePhase3
-                currentPhase1
-                currentPhase2
-                currentPhase3
+                currentL1
+                currentL2
+                currentL3
                 lastMeterConsumption
                 lastMeterProduction
+                signalStrength
             }
            }
         """
             % self.home_id
         )
-        sub_query = print_ast(document)
 
         self._subscription_id = await self._tibber_control.sub_manager.subscribe(
-            sub_query, async_callback
+            document, async_callback
         )
 
     async def rt_unsubscribe(self):
@@ -554,14 +559,13 @@ class TibberHome:
             and self._subscription_id is not None
         )
 
-    async def get_historic_data(self, n_data):
+    async def get_historic_data(self, n_data, resolution=RESOLUTION_HOURLY):
         """Get historic data."""
-        query = gql(
-            """
+        query = """
                 {
                   viewer {
                     home(id: "%s") {
-                      consumption(resolution: HOURLY, last: %s) {
+                      consumption(resolution: %s, last: %s) {
                         nodes {
                           from
                           totalCost
@@ -571,8 +575,10 @@ class TibberHome:
                     }
                   }
                 }
-          """
-            % (self.home_id, n_data)
+          """ % (
+            self.home_id,
+            resolution,
+            n_data,
         )
         data = await self._tibber_control.execute(query)
         if not data:
@@ -584,13 +590,80 @@ class TibberHome:
             return
         self._data = data["nodes"]
 
-    def sync_get_historic_data(self, n_data):
-        """get_historic_data."""
+    def sync_get_historic_data(self, n_data, resolution=RESOLUTION_HOURLY):
+        """get historic data."""
         loop = asyncio.get_event_loop()
-        task = loop.create_task(self.get_historic_data(n_data))
+        task = loop.create_task(self.get_historic_data(n_data, resolution))
         loop.run_until_complete(task)
         return self._data
 
+    def current_price_data(self):
+        """get current price."""
+        now = dt.datetime.now(self._tibber_control.time_zone)
+        for key, price_total in self.price_total.items():
+            price_time = parse(key).astimezone(self._tibber_control.time_zone)
+            price_total = round(price_total, 3)
+            time_diff = (now - price_time).total_seconds() / 60
+            if not self.last_data_timestamp or price_time > self.last_data_timestamp:
+                self.last_data_timestamp = price_time
+            if 0 <= time_diff < 60:
+                return price_total, self.price_level[key], price_time
+        return None, None, None
+
+    def current_attributes(self):
+        """get current attributes."""
+        # pylint: disable=too-many-locals
+        max_price = 0
+        min_price = 10000
+        sum_price = 0
+        off_peak_1 = 0
+        peak = 0
+        off_peak_2 = 0
+        num1 = 0
+        num0 = 0
+        num2 = 0
+        num = 0
+        now = dt.datetime.now(self._tibber_control.time_zone)
+        for key, price_total in self.price_total.items():
+            price_time = parse(key).astimezone(self._tibber_control.time_zone)
+            price_total = round(price_total, 3)
+            if now.date() == price_time.date():
+                max_price = max(max_price, price_total)
+                min_price = min(min_price, price_total)
+                if price_time.hour < 8:
+                    off_peak_1 += price_total
+                    num1 += 1
+                elif price_time.hour < 20:
+                    peak += price_total
+                    num0 += 1
+                else:
+                    off_peak_2 += price_total
+                    num2 += 1
+                num += 1
+                sum_price += price_total
+
+        attr = {}
+        attr["max_price"] = max_price
+        attr["avg_price"] = round(sum_price / num, 3) if num > 0 else 0
+        attr["min_price"] = min_price
+        attr["off_peak_1"] = round(off_peak_1 / num1, 3) if num1 > 0 else 0
+        attr["peak"] = round(peak / num0, 3) if num0 > 0 else 0
+        attr["off_peak_2"] = round(off_peak_2 / num2, 3) if num2 > 0 else 0
+        if (
+            "glitre"
+            in self.info["viewer"]["home"]["meteringPointData"]["gridCompany"].lower()
+        ):
+            now = now.astimezone(pytz.timezone("Europe/Oslo"))
+            if now.month >= 10 or now.month <= 3:
+                grid_price = 49.70 / 100
+            else:
+                grid_price = 47.45 / 100
+            if now.month >= 11 or now.month <= 3 and now.hour >= 22 or now.hour <= 7:
+                grid_price -= 14 / 100
+            attr["grid_price"] = round(grid_price, 3)
+
+        return attr
+
 
 class InvalidLogin(Exception):
     """Invalid login exception."""
